/***************************************************************************
                          CdboTracksEditor.cpp  -  description
                             -------------------
    begin                : Tue Jan 7 2003
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdatetimeedit.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtextstream.h>
#include <qfileinfo.h>
#include <qvalidator.h>
#include <qtabwidget.h>
#include <qtextedit.h>
#include <qcheckbox.h>

#include <klocale.h>
#include <klineedit.h>
#include <klistview.h>
#include <kiconloader.h>
#include <kmimetype.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kapplication.h>

#include "CdboTracksEditor.h"
#include "CdboTracksEditor.moc"

#include "CdboAudioListView.h"

#define NAME_FIELD 0
#define PATH_FIELD 1

#define TRACK_NUM_FIELD 0
#define STARTS_AT_FIELD 1

#define TITLE_FIELD 2
#define ARTIST_FIELD 3
#define LENGTH_FIELD 4

#define COUNTER_FIELD 5

#define SILENCE_FIELD 6
#define PREGAP_FIELD 7

#define COPY_PROTECTED 8
#define PREEMP 9
#define COMPOSER 10
#define SONGWRITER 11
#define ARRANGER 12
#define ISRC 13
#define MESSAGE 14


CdboTracksEditor::CdboTracksEditor( QWidget* parent, const char* name, bool modal, WFlags fl )
		: CdboTracksEditorBase( parent, name, modal, fl ){
	
	init();
	
}

CdboTracksEditor::~CdboTracksEditor(){
	delete catValidator;
}
//=======================================
void CdboTracksEditor::init(){
	defaultPregap = "0:2";
	defaultSilence = "0:2";
	tracksLst->setSorting (COUNTER_FIELD, false);
	divideSpn->setDisplay(QTimeEdit::Minutes|QTimeEdit::Seconds);
	lengthSpn->setDisplay(QTimeEdit::Minutes|QTimeEdit::Seconds);
	startsSpn->setDisplay(QTimeEdit::Minutes|QTimeEdit::Seconds);
	silenceSpn->setDisplay(QTimeEdit::Minutes|QTimeEdit::Seconds);
	pregapSpn->setDisplay(QTimeEdit::Minutes|QTimeEdit::Seconds);
	

	QRegExp rx( "[1-9]\\d{0,13}" );
	catValidator = new QRegExpValidator( rx, this,  "catValidator");
	catalogTxt->setValidator(catValidator);
}
//=======================================
bool CdboTracksEditor::loadToc(const QString &tocName, const QString &title, const QString &artist){
	tocFile = tocName;
	trackCount = 0;
	positionTracker = 0;
	lastSelected = 0;
	CDTitle = "";
	CDPerformer = "";
	
	QFile toc(tocFile);
	if(!toc.open(IO_ReadOnly)) {
		KMessageBox::error(this,
			i18n("Failed to open tracks layout description file."),
			i18n("Invalid layout descriptor."));
		return false;
	}
	
	QStringList header;
	bool headerDone = false;
	
	QStringList tracks;
	QString track, line;
	bool nextTrack = false;
	
	QTextStream inStream( &toc );
	while ( !inStream.eof() ) {
		line = inStream.readLine().stripWhiteSpace();
			
		if(line.startsWith("TRACK")){
			nextTrack = true;
			headerDone = true;
		}
			
		if(!headerDone){
			header.append(line + "\n"); 
			continue;
		}
		
		if(nextTrack){
			if(!track.isEmpty())
				tracks.append(track);
			nextTrack = false;
			track = "";
		}
		track += line + " \n";
	}
	if(!track.isEmpty())
		tracks.append(track);
	
	toc.close();

	QString catalog = "";
	if(!loadTocHeader(header, CDTitle, CDPerformer, catalog)){
		KMessageBox::error(this,
			i18n("Failed to load tracks layout from description file. "
				"Please make sure that it is in proper format and contains valid audio data."),
			i18n("Invalid layout descriptor."));
		return false;
	}
	loadTocTracks(tracks);
	
	catalogTxt->setText(catalog);
	if(!title.isNull())
		CDTitle = title;
	if(!artist.isNull())
		CDPerformer = artist;
		
	updateAlbum(CDTitle, CDPerformer);
	fixTrackCount();
	updateTotalTracks();
	
	tracksLst->setSelected(tracksLst->firstChild()->firstChild(), true);
	return true;
}
//=======================================
bool CdboTracksEditor::isValidToc(const QString &tocName, QString &title, QString &artist){
	QFile toc(tocName);
	if(!toc.open(IO_ReadOnly)) {
		KMessageBox::error(this,
			i18n("Failed to open tracks layout description file."),
			i18n("Invalid layout descriptor."));
		return false;
	}
	
	QStringList header;
	QString line;
	
	QTextStream inStream( &toc );
	while ( !inStream.eof() ) {
		line = inStream.readLine().stripWhiteSpace();
		if(line.startsWith("TRACK"))
			break;
		header.append(line + "\n"); 
	}
	
	toc.close();

	QString catalog;
	if(!loadTocHeader(header, title, artist, catalog)){
		KMessageBox::error(this,
			i18n("Failed to load tracks layout from description file. "
				"Please make sure that it is in proper format and contains valid audio data."),
			i18n("Invalid layout descriptor."));
			title = "";
			artist = "";
		return false;
	}
	return true;
}
//=======================================
bool CdboTracksEditor::loadTocHeader(QStringList &header, QString &title, QString &artist, QString &catalog){
	if(header.count() == 0)
		return false;
	
	//this supports only one language, will grab the fisrt title, etc. it finds.
	//only pure audio tocs are accepted
	
	bool valid = false;
	bool gotTitle = false, gotArtist = false;
	QString line;
	
	for(uint i = 0; i < header.count(); i++){
		line = header[i].stripWhiteSpace();
		
		if(line.startsWith("CD_DA")){
			valid = true;
			continue;
		}
		if(line.startsWith("CD_ROM")){//CD_ROM_XA should be picked as well
			valid = false;
			continue;
		}
		if(line.startsWith("CATALOG")){
			line = line.section("\"", 1,1);
			catalog = line;
			continue;
		}
		if(line.find("TITLE") != -1 && !gotTitle){
			title = line.section("\"", 1,1);
			gotTitle = true;
			continue;
		}
		if(line.find("PERFORMER") != -1 && !gotArtist){
			artist = line.section("\"", 1,1);
			gotArtist = true;
			continue;
		}
	}

	return valid;
}
//=======================================
void CdboTracksEditor::loadTocTracks(QStringList &tracks){
	//this supports only one language, will grab the fisrt title, etc. it finds.
	
	bool gotTitle = false, 
		gotArtist = false,
		gotComposer = false,
		gotWriter = false,
		gotArranger = false,
		gotIsrc = false,
		gotMessage = false;
		
	QString line;
	QStringList track;
	QListViewItem *item, *entry, *after;
	
		
	for(uint i = 0; i < tracks.count(); i++){
		line = tracks[i];
		

		line = line.right(line.length() - line.findRev("FILE") - 6);
		line = line.left(line.find("\"")).stripWhiteSpace();
		after = 0;
		entry = getEntry(line);
		if(!entry)
			entry = createEntry(line, false);
		else{
			after = entry->firstChild();
			if(after){
				while(after->nextSibling())
					after = after->nextSibling();
			}
		}
		item = new QListViewItem(entry, after);
		item->setPixmap(NAME_FIELD,BarIcon("cdtrack", KIcon::SizeSmall ));
		
		track = QStringList::split("\n", tracks[i], false);
		gotTitle = false;
		gotArtist = false;
		gotComposer = false;
		gotWriter = false;
		gotArranger = false;
		gotIsrc = false;
		gotMessage = false;
		
		for(uint i = 0; i < track.count(); i++){
			line = track[i].simplifyWhiteSpace();
			
			if(line.find("PERFORMER") != -1 && !gotArtist){
				item->setText(ARTIST_FIELD, line.section("\"", 1,1));
				gotArtist = true;
				continue;
			}
			if(line.find("TITLE") != -1 && !gotTitle){
				item->setText(TITLE_FIELD, line.section("\"", 1,1));
				gotTitle = true;
				continue;
			}

			if(line.find("COMPOSER") != -1 && !gotComposer){
				item->setText(COMPOSER, line.section("\"", 1,1));
				gotComposer = true;
				continue;
			}
			if(line.find("SONGWRITER") != -1 && !gotWriter){
				item->setText(SONGWRITER, line.section("\"", 1,1));
				gotWriter = true;
				continue;
			}
			if(line.find("ARRANGER") != -1 && !gotArranger){
				item->setText(ARRANGER, line.section("\"", 1,1));
				gotArranger = true;
				continue;
			}
			if(line.find("ISRC") != -1 && !gotIsrc){
				item->setText(ISRC, line.section("\"", 1,1));
				gotIsrc = true;
				continue;
			}
			if(line.find("MESSAGE") != -1 && !gotMessage){
				item->setText(MESSAGE, line.section("\"", 1,1));
				gotMessage = true;
				continue;
			}
			
			if(line.startsWith("PRE_EMPHASIS")){
				item->setText(PREEMP, "TRUE");
				continue;
			}
			if(line.startsWith("NO PRE_EMPHASIS")){
				item->setText(PREEMP, "FALSE");
				continue;
			}
			if(line.startsWith("NO COPY")){
				item->setText(COPY_PROTECTED, "TRUE");
				continue;
			}
			if(line.startsWith("COPY")){
				item->setText(COPY_PROTECTED, "FALSE");
				continue;
			}
			
			if(line.find("SILENCE") != -1){
				line = line.section(" ", 1,1);
				item->setText(SILENCE_FIELD, line.section(":", 0,1));
				continue;
			}
			if(line.find("START") != -1){
				line = line.section(" ", 1,1);
				item->setText(PREGAP_FIELD, line.section(":", 0,1));
				continue;
			}
			if(line.find("FILE") != -1){ 
				line = line.right(line.length() - line.findRev("\"") - 2);
				QString time = line.left(line.find(" "));
				
				if(time.contains(":") > 1)
					time = time.section(":", 0,1);
				
				item->setText(STARTS_AT_FIELD, time);
				
				line = line.right(line.length() - line.findRev(" ")).stripWhiteSpace();
				if(line.contains(":") > 1)
					line = line.section(":", 0,1);
				item->setText(LENGTH_FIELD, line);
				continue;
			}
		}
	}
}
//=======================================
bool CdboTracksEditor::copyToc(const QString &oldTocName, const QString &newTocName, bool updatePath, 
			const QString &title, const QString &artist){
	
	QFile toc(oldTocName);
	QString basePath;
	if(updatePath)
		basePath = QFileInfo(toc).dirPath(true) + "/";
	
	QFile newToc(newTocName);

	if(!toc.open(IO_ReadOnly)) {
		KMessageBox::error(this,
			i18n("Failed to open tracks layout description file."),
			i18n("Invalid layout descriptor."));
		return false;
	}
	if(!newToc.open(IO_WriteOnly)) {
		KMessageBox::error(this,
			i18n("Failed to create tracks layout description file."),
			i18n("Invalid layout descriptor."));
		toc.close();
		return false;
	}
	QString line, buff, catalogID = "";
	bool gotCatalog = false;
	bool gotHeader = false;

	QTextStream inStream( &toc ), outStream( &newToc );
	while ( !inStream.eof() ) {
		line = inStream.readLine();
		if(!gotHeader){
			if(line.startsWith("TRACK")){
				createTocHeader(outStream, title, artist, catalogID);
				gotHeader = true;
				outStream << "\n\n// Track 1\n";
				outStream << line + "\n";
				continue;
			}

			if(line.startsWith("CATALOG")){
				catalogID = line.section("\"", 1,1);
				gotCatalog = true;
				continue;
			}
			continue;
		}
		
		
		if(line.startsWith("FILE") && updatePath){
			buff = line.section("\"", 1,1);
			if(!buff.startsWith("/"))
				buff = basePath + buff;
			line = line.section("\"", 0,0) + "\"" + buff + line.right(line.length() - line.findRev("\""));
		}
		outStream << line + "\n";
	}

	toc.close();
	newToc.close();
	return true;
}
//=======================================
bool CdboTracksEditor::createToc(){
	QString catalogID = catalogTxt->text();
	if(catalogID.length() > 0 && catalogID.length() < 13){
		KMessageBox::error(this,
			i18n("Invalid Catalog ID. It must contain exactly 13 digits or can be left blank."),
			i18n("Invalid Catalog ID."));
		return false;
	}
	
	QFile file(tocFile);
	if(file.exists())
		file.remove();
	if(!file.open(IO_WriteOnly)){
		KMessageBox::error(this,
			i18n("Failed to save tracks layout. Please check working directory settings."),
			i18n("Failed to save."));
		return false;
	}
	QTextStream stream(&file);
	createTocHeader(stream, CDTitle, CDPerformer, catalogID);

	bool copy_protected, preemphasis; 
	for(QListViewItem *item = tracksLst->firstChild(); item; item = item->nextSibling()){
		for(QListViewItem *item2 = item->firstChild(); item2; item2 = item2->nextSibling()){

			copy_protected = item2->text(COPY_PROTECTED) == "TRUE"; 
			preemphasis = item2->text(PREEMP) == "TRUE"; 
			createTocEntry(stream, item2->text(TRACK_NUM_FIELD), item->text(PATH_FIELD), item2->text(STARTS_AT_FIELD)+":0", 
				item2->text(LENGTH_FIELD)+":0", item2->text(SILENCE_FIELD)+":0", item2->text(PREGAP_FIELD)+":0", 
				item2->text(TITLE_FIELD), item2->text(ARTIST_FIELD), copy_protected, preemphasis, 
				item2->text(SONGWRITER), item2->text(COMPOSER), item2->text(ARRANGER), item2->text(ISRC), item2->text(MESSAGE));
		
		}
	}
	
	file.close();
	return true;
}
//=======================================
bool CdboTracksEditor::createDefaultToc(QString fileList, const QString &tocName, const QString &delim, 
			const QString &title, const QString &artist){
			
	if(fileList.stripWhiteSpace().length() == 0)
		return false;

	QString catalogID = "";

	QFile file(tocName);
	if(file.exists())
		file.remove();
	if(!file.open(IO_ReadWrite))
		return false;
	
	QTextStream stream(&file);
	createTocHeader(stream, title, artist, catalogID);

	QStringList list = QStringList::split(delim, fileList, false);
	
	for(uint i = 0; i < list.count(); i++){
		createTocEntry(stream, i18n("Track %1").arg(QString::number(i+1)), list[i], "0", "", "0:2:0", "0:2:0");
	}
	
	file.close();
	return true;
}
//=======================================
void CdboTracksEditor::createTocHeader(QTextStream &stream, const QString &title, const QString &artist, const QString &catalogID){
	stream << "CD_DA\n\n";
	
	if(!catalogID.isEmpty())
			stream << "CATALOG \"" << catalogID << "\"\n\n";
	
	stream << i18n("//cdrdao style toc file created by CD Bake Oven. ");
	stream << QDate::currentDate().toString() << ".\n\n";
	
	stream << "CD_TEXT {\n\tLANGUAGE_MAP {\n\t\t0 : EN\n\t}\n";
			
	stream << "\tLANGUAGE 0 {\n";
	if(!title.isEmpty())
		stream << "\t\tTITLE \"" << title << "\"\n";
	if(!artist.isEmpty())
		stream << "\t\tPERFORMER \"" << artist << "\"\n";
	stream << "\t}\n";
	
	stream << "}\n";
}
//=======================================
void CdboTracksEditor::createTocEntry(QTextStream &stream, const QString &track, const QString &file, const QString &starts, 
		const QString &length, const QString &silence, const QString &pregap, const QString &title, const QString &artist, bool copy_protected, 
		bool preemphasis, const QString &writer, const QString &composer, const QString &arranger, const QString &isrc, const QString &message){

	stream << "\n\n// " << track << "\n";
	stream << "TRACK AUDIO\n";
	if(copy_protected)
		stream << "NO COPY\n";
	else
		stream << "COPY\n";
	if(preemphasis)
		stream << "PRE_EMPHASIS\n";
	else
		stream << "NO PRE_EMPHASIS\n";

	
	stream << "CD_TEXT {\n\tLANGUAGE 0 {\n";
	
	if(!title.isEmpty())
		stream << "\t\tTITLE \"" << title << "\"\n";
	if(!artist.isEmpty())
		stream << "\t\tPERFORMER \"" << artist << "\"\n";
	if(!writer.isEmpty())
		stream << "\t\tSONGWRITER \"" << writer << "\"\n";
	if(!composer.isEmpty())
		stream << "\t\tCOMPOSER \"" << composer << "\"\n";
	if(!arranger.isEmpty())
		stream << "\t\tARRANGER \"" << arranger << "\"\n";
	if(!isrc.isEmpty())
		stream << "\t\tISRC \"" << isrc << "\"\n";
	if(!message.isEmpty())
		stream << "\t\tMESSAGE \"" << message << "\"\n";
	
	stream << "\t}\n}\n";

	stream << "SILENCE " << silence << "\n";
	stream << "FILE \"" << file << "\" " << starts << " " << length << "\n";
	stream << "START " << pregap << "\n";
}
//=======================================
void CdboTracksEditor::loadFileList(QString fileList, const QString &tocName, const QString &delim,
			const QString &title, const QString &artist){
	if(fileList.stripWhiteSpace().length() == 0)
		return;
	
	tocFile = tocName;
	CDTitle = artist;
	CDPerformer = title;
	
	trackCount = 0;
	positionTracker = 0;
	lastSelected = 0;
	
	updateAlbum(title, artist);
	
	QStringList tracks = QStringList::split(delim , fileList, false);
	if(tracks.count() == 0)
		return;
	for(uint i = 0; i < tracks.count(); i++)
		createEntry(tracks[i].stripWhiteSpace());
	tracksLst->setSelected(tracksLst->firstChild()->firstChild(), true);
	updateTotalTracks();
}
//=======================================
QListViewItem *CdboTracksEditor::createEntry(const QString &path, bool withDefaultTrack){
	positionTracker++;
	
	QString title, artist, length;
	CdboAudioListView::getMetaInfo(path, title, artist, length);
	QString fileName = path.right(path.length() - path.findRev("/") - 1);
	
	QListViewItem *item = new QListViewItem(tracksLst, tracksLst->lastChild(), fileName.stripWhiteSpace(), path.stripWhiteSpace(), 
		title.stripWhiteSpace(), artist.stripWhiteSpace(), length.stripWhiteSpace(),
		positionTracker > 9? QString::number(positionTracker): "0" + QString::number(positionTracker));
	item->setPixmap(NAME_FIELD,BarIcon(KMimeType::iconForURL(path), KIcon::SizeSmall ));
	
	if(withDefaultTrack)
		createTrack(item);
	
	item->setOpen(true);
	return item;
}
//=======================================
QListViewItem *CdboTracksEditor::getEntry(const QString &path){
	for(QListViewItem* item = tracksLst->firstChild(); ; item = item->nextSibling()){
		if(!item)
			return 0;
		if(item->text(PATH_FIELD) == path)
			return item;
	}
	return 0;
}
//=======================================
void CdboTracksEditor::createTrack(QListViewItem *parentItem){
	trackCount++;
	positionTracker++;
	
	QString title = parentItem->text(TITLE_FIELD);
	if(title == i18n("Unknown"))
		title = "";
	
	QString artist = parentItem->text(ARTIST_FIELD);
	if(artist == i18n("Unknown"))
		artist = "";
	
	QListViewItem *item = new QListViewItem(parentItem, i18n("Track %1").arg(QString::number(trackCount)), "0:0",
		title, artist, parentItem->text(LENGTH_FIELD),
		positionTracker > 9? QString::number(positionTracker): "0" + QString::number(positionTracker),
		defaultSilence, defaultPregap);
	item->setPixmap(NAME_FIELD,BarIcon("cdtrack", KIcon::SizeSmall ));
	
	item->setText(COPY_PROTECTED,"FALSE");
	item->setText(PREEMP,"FALSE");
	item->setText(COMPOSER,"");
	item->setText(SONGWRITER,"");
	item->setText(ARRANGER,"");
	item->setText(ISRC,"");
	item->setText(MESSAGE,"");
}
//=======================================
void CdboTracksEditor::selectionChangedSlot(QListViewItem* item){
	updateLastSelected();
	QListViewItem *item2 = item->parent();
	if(!item2){
		initMetaInfo(item);
		trackInfoPnl->hide();
		lastSelected = 0;
	}
	else{
		initMetaInfo(item2);
		lastSelected = item;
		initTrackInfo(item);
		trackInfoPnl->show();
	}
}
//=======================================
void CdboTracksEditor::initMetaInfo(QListViewItem* item){
	titleMetaInfoLbl->setText(i18n("Title: %1").arg(item->text(TITLE_FIELD)));
	artistMetaInfoLbl->setText(i18n("Artist: %1").arg(item->text(ARTIST_FIELD)));
	lengthMetaInfoLbl->setText(i18n("Length: %1").arg(item->text(LENGTH_FIELD)));
}
//=======================================
void CdboTracksEditor::initTrackInfo(QListViewItem* item){
	trackInfoPnl->changeTab(detailsTab, i18n("%1 Detai&ls").arg(item->text(TRACK_NUM_FIELD)));
	artistTxt->setText(item->text(ARTIST_FIELD));
	titleTxt->setText(item->text(TITLE_FIELD));
	
	composerTxt->setText(item->text(COMPOSER));
	writerTxt->setText(item->text(SONGWRITER));
	arrangerTxt->setText(item->text(ARRANGER));
	isrcTxt->setText(item->text(ISRC));
	messageTxt->setText(item->text(MESSAGE));
	
	copyProtectedChk->setChecked(item->text(COPY_PROTECTED) == "TRUE");
	preempChk->setChecked(item->text(PREEMP) == "TRUE");
	
	
	QString length = item->text(LENGTH_FIELD);
	QString l = length;
	
	if(!updateTime(lengthSpn, length)){
		lengthSpn->setMaxValue(QTime(23, 59, 59));
		updateTime(lengthSpn, "0:0");
		startsSpn->setMaxValue(QTime(23, 59, 59));
		divideSpn->setMaxValue(QTime(23, 59, 59));
	}
	else{
		bool ok;
		length = item->parent()->text(LENGTH_FIELD);
		int m = length.left(length.find(":")).toInt(&ok);
		int h = 0;
		if(m > 59){
			h = m / 60;
			m = m % 60;
		}
		int s = length.right(length.length() - length.findRev(":") - 1).toInt(&ok);
		lengthSpn->setMaxValue(QTime(h, m, s));
		updateTime(lengthSpn, l);//for some reason have to do it twice ........
		startsSpn->setMaxValue(QTime(h, m, s-4));//must be at least 4 secs
		divideSpn->setMaxValue(QTime(h, m, s));
	}
	
	updateTime(silenceSpn, item->text(SILENCE_FIELD));	
	updateTime(pregapSpn, item->text(PREGAP_FIELD));
	updateTime(startsSpn, item->text(STARTS_AT_FIELD));

	if(item == item->parent()->firstChild())
		removeBtn->setEnabled(false);
	else
		removeBtn->setEnabled(true);
}
//=======================================
bool CdboTracksEditor::updateTime(QTimeEdit * edit, QString length){
	bool ok;
	int m = length.left(length.find(":")).toInt(&ok);
	if(!ok)
		return false;
	int h = 0;
	if(m > 59){
		h = m / 60;
		m = m % 60;
	}
	int s = length.right(length.length() - length.findRev(":") - 1).toInt(&ok);
	if(!ok)
		return false;
	edit->setTime(QTime(h, m, s));
	return true;
}
//=======================================
void CdboTracksEditor::updateLastSelected(){
	if(!lastSelected)
		return;
	lastSelected->setText(ARTIST_FIELD, artistTxt->text());
	lastSelected->setText(TITLE_FIELD, titleTxt->text());
	
	lastSelected->setText(COMPOSER, composerTxt->text());
	lastSelected->setText(SONGWRITER, writerTxt->text());
	lastSelected->setText(ARRANGER, arrangerTxt->text());
	lastSelected->setText(ISRC, isrcTxt->text());
	lastSelected->setText(MESSAGE, messageTxt->text());
	
	lastSelected->setText(COPY_PROTECTED, copyProtectedChk->isChecked()? "TRUE": "FALSE");
	lastSelected->setText(PREEMP, preempChk->isChecked()? "TRUE": "FALSE");
	
	int m = lengthSpn->time().hour() * 60 + lengthSpn->time().minute();
	int s = lengthSpn->time().second();
	lastSelected->setText(LENGTH_FIELD, QString::number(m) + ":" + QString::number(s));
	
	m = pregapSpn->time().hour() * 60 + pregapSpn->time().minute();
	s = pregapSpn->time().second();
	lastSelected->setText(PREGAP_FIELD, QString::number(m) + ":" + QString::number(s));
	
	m = silenceSpn->time().hour() * 60 + silenceSpn->time().minute();
	s = silenceSpn->time().second();
	lastSelected->setText(SILENCE_FIELD, QString::number(m) + ":" + QString::number(s));
	
	
	m = startsSpn->time().hour() * 60 + startsSpn->time().minute();
	s = startsSpn->time().second();
	lastSelected->setText(STARTS_AT_FIELD, QString::number(m) + ":" + QString::number(s));
	
}
//=======================================
void CdboTracksEditor::startTimeChangedSlot(const QTime& newTime){
	if(!lastSelected)
		return;
	bool ok;
	QListViewItem* parentItem = lastSelected->parent();
	QString length = parentItem->text(LENGTH_FIELD);
	int m = length.left(length.find(":")).toInt(&ok);
	if(!ok)
		return;

	int s = length.right(length.length() - length.findRev(":") - 1).toInt(&ok);
	if(!ok)
		return;
	int ss = newTime.hour() * 600;
	ss += newTime.minute() * 60; 
	ss += newTime.second();
	//parent time should always be bigger
	s = (m * 60 + s) - ss;
	int h = 0;
	m = 0;
	if(s > 599){
		h = s / 600;
		s = s % 600;
	}
	if(s > 59){
		m = s / 60;
		s = s % 60;
	}
	lengthSpn->setMaxValue(QTime(h,m,s));
	if(lengthSpn->maxValue() < lengthSpn->time())
		lengthSpn->setTime(lengthSpn->maxValue());
}
//=======================================
void CdboTracksEditor::divideSlot(){
	if(!lastSelected)
		return;

	QListViewItem* parentItem = lastSelected->parent();
	int m = divideSpn->time().hour() * 60 + divideSpn->time().minute();
	int s = divideSpn->time().second();
	
	bool ok;
	QString tn = lastSelected->text(TRACK_NUM_FIELD);
	trackCount = tn.right(tn.length() - tn.findRev(" ") - 1).toInt(&ok) + 1;

	QListViewItem *item = new QListViewItem(parentItem, lastSelected, i18n("Track %1").arg(QString::number(trackCount)), 
		QString::number(m) + ":" + QString::number(s), parentItem->text(TITLE_FIELD), 
		parentItem->text(ARTIST_FIELD), parentItem->text(LENGTH_FIELD),
		positionTracker > 9? QString::number(positionTracker): "0" + QString::number(positionTracker),
		defaultSilence, defaultPregap);

	item->setPixmap(NAME_FIELD,BarIcon("cdtrack", KIcon::SizeSmall ));
	
	
	fixTrackCount();
	
	tracksLst->setSelected(item, true);
	updateTotalTracks();
}
//=======================================
void CdboTracksEditor::removeSlot(){
	if(!lastSelected)
		return;
	QListViewItem *item = lastSelected->itemAbove();
	delete lastSelected;
	lastSelected = 0;
	
	fixTrackCount();
	
	tracksLst->setSelected(item, true);
	updateTotalTracks();
}
//=======================================
void CdboTracksEditor::fixTrackCount(){
	trackCount = 1;
	for(QListViewItem *item = tracksLst->firstChild(); ; item = item->nextSibling()){
		if(!item)
			return;
		for(QListViewItem *item2 = item->firstChild(); ; item2 = item2->nextSibling()){
			if(!item2)
				break;
			item2->setText(TRACK_NUM_FIELD, i18n("Track %1").arg(QString::number(trackCount)));
			trackCount++;
		}
	}
}
//=======================================
void CdboTracksEditor::updateTotalTracks(){
	QListViewItem* item = tracksLst->lastChild();
	for(item = item->firstChild(); ; item = item->nextSibling()){
		if(!item->nextSibling())
			break;
	}
	
	QString tn = item->text(TRACK_NUM_FIELD);
	totalTracksLbl->setText(i18n("Total Tracks: %1").arg(tn.right(tn.length() - tn.findRev(" ") - 1)));
}
//=======================================
void CdboTracksEditor::updateAlbum(const QString &title, const QString &artist){
	if(title.isEmpty())
		albumLbl->setText(i18n("Untitled"));
	else
		albumLbl->setText(title);
		
	if(artist.isEmpty())
		albumArtistLbl->setText(i18n("Unknown"));
	else
		albumArtistLbl->setText(artist);
}
//=======================================
void CdboTracksEditor::accept(){
	updateLastSelected();
	if(!createToc())
		return;
	QDialog::accept();
}
//=======================================
void CdboTracksEditor::helpSlot(){
	kapp->invokeHelp("", "cdbakeoven");
}


